@@ -13,9 +13,8 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +23,12 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <terminal/terminal-dialogs.h>
+#ifdef GDK_WINDOWING_X11
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#endif
+
+#include <terminal/terminal-util.h>
 #include <terminal/terminal-enum-types.h>
 #include <terminal/terminal-preferences-dialog.h>
 #include <terminal/terminal-encoding-action.h>
@@ -36,6 +40,13 @@ static void terminal_preferences_dialog_finalize          (GObject
 static void terminal_preferences_dialog_response          (GtkWidget                 *widget,
                                                            gint                       response,
                                                            TerminalPreferencesDialog *dialog);
+#ifdef GDK_WINDOWING_X11
+static void terminal_preferences_dialog_geometry_changed  (TerminalPreferencesDialog *dialog);
+static void terminal_preferences_dialog_geometry_columns  (GtkAdjustment             *adj,
+                                                           TerminalPreferencesDialog *dialog);
+static void terminal_preferences_dialog_geometry_rows     (GtkAdjustment             *adj,
+                                                           TerminalPreferencesDialog *dialog);
+#endif
 static void terminal_preferences_dialog_palette_changed   (GtkWidget                 *button,
                                                            TerminalPreferencesDialog *dialog);
 static void terminal_preferences_dialog_palette_notify    (TerminalPreferencesDialog *dialog);
@@ -70,6 +81,7 @@ struct _TerminalPreferencesDialog
 
   gulong               bg_image_signal_id;
   gulong               palette_signal_id;
+  gulong               geometry_sigal_id;
 };
 
 enum
@@ -131,12 +143,19 @@ terminal_preferences_dialog_init (TerminalPreferencesDialog *dialog)
                                        "shortcuts-no-mnemonics", "shortcuts-no-menukey",
                                        "binding-backspace", "binding-delete",
                                        "background-mode", "background-image-style",
-                                       "color-background-vary"
+                                       "color-background-vary", "dropdown-keep-open-default",
+                                       "dropdown-keep-above", "dropdown-toggle-focus",
+                                       "dropdown-status-icon", "dropdown-move-to-active",
+                                       "dropdown-always-show-tabs"
                                      };
   const gchar      *props_color[] =  { "color-foreground", "color-cursor",
                                        "color-background", "tab-activity-color",
                                        "color-selection", "color-bold"
                                      };
+  const gchar      *props_value[] =  { "dropdown-height", "dropdown-width",
+                                       "dropdown-position", "dropdown-opacity",
+                                       "dropdown-animation-time"
+                                     };
 
   dialog->preferences = terminal_preferences_get ();
 
@@ -179,6 +198,10 @@ error:
   for (i = 0; i < G_N_ELEMENTS (props_color); i++)
     BIND_PROPERTIES (props_color[i], "color");
 
+  /* bind color properties */
+  for (i = 0; i < G_N_ELEMENTS (props_value); i++)
+    BIND_PROPERTIES (props_value[i], "value");
+
   /* bind color palette properties */
   for (i = 1; i <= 16; i++)
     {
@@ -200,7 +223,7 @@ error:
   /* other properties */
   BIND_PROPERTIES ("font-name", "font-name");
   BIND_PROPERTIES ("title-initial", "text");
-  BIND_PROPERTIES ("term", "text");
+  BIND_PROPERTIES ("emulation", "text");
   BIND_PROPERTIES ("word-chars", "text");
   BIND_PROPERTIES ("scrolling-lines", "value");
   BIND_PROPERTIES ("tab-activity-timeout", "value");
@@ -218,6 +241,12 @@ error:
   g_signal_connect (G_OBJECT (object), "clicked",
       G_CALLBACK (terminal_preferences_dialog_reset_word_chars), dialog);
 
+  /* position scale */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "scale-position");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  for (i = 0; i <= 100; i += 25)
+    gtk_scale_add_mark (GTK_SCALE (object), i, GTK_POS_BOTTOM, NULL);
+
   /* inverted custom colors and set sensitivity */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "color-selection-custom");
   terminal_return_if_fail (G_IS_OBJECT (object));
@@ -239,6 +268,28 @@ error:
                           G_OBJECT (object2), "sensitive",
                           G_BINDING_SYNC_CREATE);
 
+#ifdef GDK_WINDOWING_X11
+  terminal_preferences_dialog_geometry_changed (dialog);
+  dialog->geometry_sigal_id = g_signal_connect_swapped (G_OBJECT (dialog->preferences),
+      "notify::misc-default-geometry",
+      G_CALLBACK (terminal_preferences_dialog_geometry_changed), dialog);
+
+  /* geo changes */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "geo-columns");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect (G_OBJECT (object), "value-changed",
+      G_CALLBACK (terminal_preferences_dialog_geometry_columns), dialog);
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "geo-rows");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect (G_OBJECT (object), "value-changed",
+      G_CALLBACK (terminal_preferences_dialog_geometry_rows), dialog);
+#else
+  /* hide */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "geo-box");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  gtk_widget_hide (GTK_BOX (object));
+#endif
+
   /* background widgets visibility */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-mode");
   terminal_return_if_fail (G_IS_OBJECT (object));
@@ -292,6 +343,8 @@ terminal_preferences_dialog_finalize (GObject *object)
     g_signal_handler_disconnect (dialog->preferences, dialog->bg_image_signal_id);
   if (G_LIKELY (dialog->palette_signal_id != 0))
     g_signal_handler_disconnect (dialog->preferences, dialog->palette_signal_id);
+  if (G_LIKELY (dialog->geometry_sigal_id != 0))
+    g_signal_handler_disconnect (dialog->preferences, dialog->geometry_sigal_id);
 
   /* release the preferences */
   g_object_unref (G_OBJECT (dialog->preferences));
@@ -306,14 +359,28 @@ terminal_preferences_dialog_response (GtkWidget                 *widget,
                                       gint                       response,
                                       TerminalPreferencesDialog *dialog)
 {
-  GSList *li;
+  GSList      *li;
+  GObject     *object;
+  GObject     *notebook;
+  const gchar *section;
 
   /* check if we should open the user manual */
   if (G_UNLIKELY (response == 1))
     {
+      /* if the drop-down preferences are shown, we open that page in the wiki */
+      notebook = gtk_builder_get_object (GTK_BUILDER (dialog), "notebook");
+      terminal_return_if_fail (GTK_IS_NOTEBOOK (notebook));
+      object = gtk_builder_get_object (GTK_BUILDER (dialog), "dropdown-box");
+      terminal_return_if_fail (G_IS_OBJECT (object));
+      if (gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (object))
+          == gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))
+        section = "dropdown";
+      else
+        section = "preferences";
+
       /* open the "Preferences" section of the user manual */
       xfce_dialog_show_help (GTK_WINDOW (widget), "terminal",
-                             "preferences", NULL);
+                             section, NULL);
     }
   else
     {
@@ -329,6 +396,104 @@ terminal_preferences_dialog_response (GtkWidget                 *widget,
 
 
 
+#ifdef GDK_WINDOWING_X11
+static void
+terminal_preferences_dialog_geometry_changed (TerminalPreferencesDialog *dialog)
+{
+  GObject *object;
+  gchar   *geo;
+  guint    w = 0, h = 0;
+  gint     x, y;
+
+  g_object_get (G_OBJECT (dialog->preferences), "misc-default-geometry", &geo, NULL);
+  if (G_LIKELY (geo != NULL))
+    {
+      /* parse the string */
+      XParseGeometry (geo, &x, &y, &w, &h);
+      g_free (geo);
+    }
+
+  /* set cols */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "geo-columns");
+  terminal_return_if_fail (GTK_IS_ADJUSTMENT (object));
+  g_signal_handlers_block_by_func (G_OBJECT (object),
+      terminal_preferences_dialog_geometry_columns, dialog);
+  gtk_adjustment_set_value (GTK_ADJUSTMENT (object), w != 0 ? w : 80);
+  g_signal_handlers_unblock_by_func (G_OBJECT (object),
+      terminal_preferences_dialog_geometry_columns, dialog);
+
+  /* set rows */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "geo-rows");
+  terminal_return_if_fail (GTK_IS_ADJUSTMENT (object));
+  g_signal_handlers_block_by_func (G_OBJECT (object),
+      terminal_preferences_dialog_geometry_columns, dialog);
+  gtk_adjustment_set_value (GTK_ADJUSTMENT (object), h != 0 ? h : 24);
+  g_signal_handlers_unblock_by_func (G_OBJECT (object),
+      terminal_preferences_dialog_geometry_columns, dialog);
+}
+
+
+
+static void
+terminal_preferences_dialog_geometry (TerminalPreferencesDialog *dialog,
+                                      gint                       columns,
+                                      gint                       rows)
+{
+  gint     x, y;
+  guint    w, h;
+  gchar   *geo;
+  gint     mask = NoValue;
+
+  g_object_get (G_OBJECT (dialog->preferences), "misc-default-geometry", &geo, NULL);
+  if (G_LIKELY (geo != NULL))
+    {
+      /* parse the string */
+      mask = XParseGeometry (geo, &x, &y, &w, &h);
+      g_free (geo);
+    }
+
+  /* set new value */
+  if (columns > 0)
+    w = columns;
+  if (rows > 0)
+    h = rows;
+
+  /* if there is an x or y value, preserve this */
+  if ((mask & XValue) != 0 || (mask & YValue) != 0)
+    geo = g_strdup_printf ("%dx%d%+d%+d", w, h, x, y);
+  else
+    geo = g_strdup_printf ("%dx%d", w, h);
+
+  /* save */
+  g_signal_handler_block (G_OBJECT (dialog->preferences), dialog->geometry_sigal_id);
+  g_object_set (G_OBJECT (dialog->preferences), "misc-default-geometry", geo, NULL);
+  g_signal_handler_unblock (G_OBJECT (dialog->preferences), dialog->geometry_sigal_id);
+  g_free (geo);
+}
+
+
+
+static void
+terminal_preferences_dialog_geometry_columns (GtkAdjustment             *adj,
+                                              TerminalPreferencesDialog *dialog)
+{
+  terminal_return_if_fail (GTK_IS_ADJUSTMENT (adj));
+  terminal_preferences_dialog_geometry (dialog, gtk_adjustment_get_value (adj), -1);
+}
+
+
+
+static void
+terminal_preferences_dialog_geometry_rows (GtkAdjustment             *adj,
+                                           TerminalPreferencesDialog *dialog)
+{
+  terminal_return_if_fail (GTK_IS_ADJUSTMENT (adj));
+  terminal_preferences_dialog_geometry (dialog, -1, gtk_adjustment_get_value (adj));
+}
+#endif
+
+
+
 static void
 terminal_preferences_dialog_palette_changed (GtkWidget                 *button,
                                              TerminalPreferencesDialog *dialog)
@@ -505,15 +670,15 @@ terminal_preferences_dialog_presets_changed (GtkComboBox               *combobox
 static void
 terminal_preferences_dialog_presets_load (TerminalPreferencesDialog *dialog)
 {
-  gchar   **presets;
-  guint     n;
-  GObject  *object;
-  guint     n_presets = 0;
-  XfceRc   *rc;
+  gchar       **presets;
+  guint         n;
+  GObject      *object;
+  guint         n_presets = 0;
+  XfceRc       *rc;
   GtkListStore *store;
-  GtkTreeIter iter;
-  const gchar *title;
-  gchar *path;
+  GtkTreeIter   iter;
+  const gchar  *title;
+  gchar        *path;
 
   /* load schemes */
   presets = xfce_resource_match (XFCE_RESOURCE_DATA, "xfce4/terminal/colorschemes/*", TRUE);
@@ -601,7 +766,7 @@ terminal_preferences_dialog_reset_compat (GtkWidget                 *button,
 {
   GParamSpec  *spec;
   GValue       value = { 0, };
-  const gchar *properties[] = { "binding-backspace", "binding-delete", "term" };
+  const gchar *properties[] = { "binding-backspace", "binding-delete", "emulation" };
   guint        i;
 
   for (i = 0; i < G_N_ELEMENTS (properties); i++)
@@ -733,20 +898,33 @@ terminal_preferences_dialog_encoding_changed (GtkComboBox               *combobo
 
 /**
  * terminal_preferences_dialog_new:
- * @parent      : A #GtkWindow or %NULL.
  *
  * Return value :
  **/
 GtkWidget*
-terminal_preferences_dialog_new (GtkWindow *parent)
+terminal_preferences_dialog_new (gboolean show_drop_down)
 {
   GtkBuilder *builder;
   GObject    *dialog;
+  GObject    *object;
+  GObject    *notebook;
 
   builder = g_object_new (TERMINAL_TYPE_PREFERENCES_DIALOG, NULL);
-  dialog = gtk_builder_get_object (builder, "dialog");
-  if (parent != NULL && dialog != NULL)
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
 
+  object = gtk_builder_get_object (builder, "dropdown-box");
+  terminal_return_val_if_fail (GTK_IS_WIDGET (object), NULL);
+  gtk_widget_set_visible (GTK_WIDGET (object), show_drop_down);
+
+  if (show_drop_down)
+    {
+      /* focus the drop-down tab if enabled */
+      notebook = gtk_builder_get_object (builder, "notebook");
+      terminal_return_val_if_fail (GTK_IS_NOTEBOOK (notebook), NULL);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
+          gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (object)));
+    }
+
+  dialog = gtk_builder_get_object (builder, "dialog");
+  terminal_return_val_if_fail (XFCE_IS_TITLED_DIALOG (dialog), NULL);
   return GTK_WIDGET (dialog);
 }
