@@ -11,9 +11,8 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +54,7 @@ struct _TerminalSearchDialog
 
   GtkWidget *button_prev;
   GtkWidget *button_next;
-  
+
   GtkWidget *entry;
 
   GtkWidget *match_case;
@@ -96,19 +95,24 @@ terminal_search_dialog_init (TerminalSearchDialog *dialog)
   dialog->button_prev = xfce_gtk_button_new_mixed (GTK_STOCK_GO_BACK, _("_Previous"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->button_prev, TERMINAL_RESPONSE_SEARCH_PREV);
   gtk_widget_set_can_default (dialog->button_prev, TRUE);
+  gtk_widget_show (dialog->button_prev);
 
   dialog->button_next = xfce_gtk_button_new_mixed (GTK_STOCK_GO_FORWARD, _("_Next"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->button_next, TERMINAL_RESPONSE_SEARCH_NEXT);
+  gtk_widget_show (dialog->button_next);
 
   vbox = gtk_vbox_new (FALSE, 6);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), vbox, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+  gtk_widget_show (vbox);
 
   hbox = gtk_hbox_new (FALSE, 12);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+  gtk_widget_show (hbox);
 
   label = gtk_label_new_with_mnemonic (_("_Search for:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
 
   dialog->entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), dialog->entry, TRUE, TRUE, 0);
@@ -119,26 +123,31 @@ terminal_search_dialog_init (TerminalSearchDialog *dialog)
       G_CALLBACK (terminal_search_dialog_entry_icon_release), NULL);
   g_signal_connect (G_OBJECT (dialog->entry), "changed",
       G_CALLBACK (terminal_search_dialog_entry_changed), dialog);
+  gtk_widget_show (dialog->entry);
 
   dialog->match_case = gtk_check_button_new_with_mnemonic (_("C_ase sensitive"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_case, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_case), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
+  gtk_widget_show (dialog->match_case);
 
   dialog->match_regex = gtk_check_button_new_with_mnemonic (_("Match as _regular expression"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_regex, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_regex), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
+  gtk_widget_show (dialog->match_regex);
 
   dialog->match_word = gtk_check_button_new_with_mnemonic (_("Match _entire word only"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_word, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_word), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
+  gtk_widget_show (dialog->match_word);
 
   dialog->wrap_around = gtk_check_button_new_with_mnemonic (_("_Wrap around"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->wrap_around, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->wrap_around), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
+  gtk_widget_show (dialog->wrap_around);
 
   terminal_search_dialog_entry_changed (dialog->entry, dialog);
 }
