@@ -13,12 +13,8 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA.
- *
- * The geometry handling code was taken from gnome-terminal. The geometry hacks
- * were initially written by Owen Taylor.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +39,7 @@
 #include <gdk/gdkx.h>
 #endif
 
-#include <terminal/terminal-dialogs.h>
+#include <terminal/terminal-util.h>
 #include <terminal/terminal-enum-types.h>
 #include <terminal/terminal-options.h>
 #include <terminal/terminal-preferences-dialog.h>
@@ -52,6 +48,7 @@
 #include <terminal/terminal-marshal.h>
 #include <terminal/terminal-encoding-action.h>
 #include <terminal/terminal-window.h>
+#include <terminal/terminal-window-dropdown.h>
 #include <terminal/terminal-window-ui.h>
 #include <terminal/terminal-widget.h>
 
@@ -87,7 +84,7 @@ static gboolean        terminal_window_accel_activate                (GtkAccelGr
                                                                       GdkModifierType         accel_mods,
                                                                       TerminalWindow         *window);
 static void            terminal_window_update_actions                (TerminalWindow         *window);
-static void            terminal_window_rebuild_gomenu                (TerminalWindow         *window);
+static void            terminal_window_rebuild_tabs_menu             (TerminalWindow         *window);
 static void            terminal_window_notebook_page_switched        (GtkNotebook            *notebook,
                                                                       GtkNotebookPage        *page,
                                                                       guint                   page_num,
@@ -96,7 +93,6 @@ static void            terminal_window_notebook_page_reordered       (GtkNoteboo
                                                                       GtkNotebookPage        *page,
                                                                       guint                   page_num,
                                                                       TerminalWindow         *window);
-static void            terminal_window_notebook_show_tabs            (TerminalWindow         *window);
 static void            terminal_window_notebook_page_added           (GtkNotebook            *notebook,
                                                                       GtkWidget              *child,
                                                                       guint                   page_num,
@@ -184,62 +180,14 @@ static void            terminal_window_action_reset_and_clear        (GtkAction
                                                                       TerminalWindow         *window);
 static void            terminal_window_action_contents               (GtkAction              *action,
                                                                       TerminalWindow         *window);
-static void            terminal_window_action_report_bug             (GtkAction              *action,
-                                                                      TerminalWindow         *window);
 static void            terminal_window_action_about                  (GtkAction              *action,
                                                                       TerminalWindow         *window);
 
 
 
-struct _TerminalWindowClass
-{
-  GtkWindowClass __parent__;
-};
-
-struct _TerminalWindow
-{
-  GtkWindow            __parent__;
-
-  TerminalPreferences *preferences;
-  GtkWidget           *preferences_dialog;
-
-  GtkActionGroup      *action_group;
-  GtkUIManager        *ui_manager;
-
-  guint                gomenu_merge_id;
-  GSList              *gomenu_actions;
-
-  GtkWidget           *menubar;
-  GtkWidget           *toolbar;
-  GtkWidget           *notebook;
-
-  GtkWidget           *search_dialog;
-
-  /* pushed size of screen */
-  glong                grid_width;
-  glong                grid_height;
-
-  GtkAction           *encoding_action;
-
-  TerminalScreen      *active;
-
-  /* cached actions to avoid lookups */
-  GtkAction           *action_detah_tab;
-  GtkAction           *action_close_tab;
-  GtkAction           *action_prev_tab;
-  GtkAction           *action_next_tab;
-  GtkAction           *action_move_tab_left;
-  GtkAction           *action_move_tab_right;
-  GtkAction           *action_copy;
-  GtkAction           *action_search_next;
-  GtkAction           *action_search_prev;
-};
-
-
-
 static guint         window_signals[LAST_SIGNAL];
 static gconstpointer window_notebook_group = PACKAGE_NAME;
-static GQuark        gomenu_action_quark = 0;
+static GQuark        tabs_menu_action_quark = 0;
 
 
 
@@ -263,16 +211,15 @@ static const GtkActionEntry action_entries[] =
     { "search", GTK_STOCK_FIND, N_ ("_Find..."), "<control><shift>f", NULL, G_CALLBACK (terminal_window_action_search), },
     { "search-next", NULL, N_ ("Find Ne_xt"), NULL, NULL, G_CALLBACK (terminal_window_action_search_next), },
     { "search-prev", NULL, N_ ("Find Pre_vious"), NULL, NULL, G_CALLBACK (terminal_window_action_search_prev), },
-    { "reset", NULL, N_ ("_Clear Scrollback"), NULL, NULL, G_CALLBACK (terminal_window_action_reset), },
-    { "reset-and-clear", NULL, N_ ("Clear Scrollback and _Reset"), NULL, NULL, G_CALLBACK (terminal_window_action_reset_and_clear), },
-  { "go-menu", NULL, N_ ("_Go"), NULL, NULL, NULL, },
+    { "reset", NULL, N_ ("_Reset"), NULL, NULL, G_CALLBACK (terminal_window_action_reset), },
+    { "reset-and-clear", NULL, N_ ("_Clear Scrollback and Reset"), NULL, NULL, G_CALLBACK (terminal_window_action_reset_and_clear), },
+  { "tabs-menu", NULL, N_ ("T_abs"), NULL, NULL, NULL, },
     { "prev-tab", GTK_STOCK_GO_BACK, N_ ("_Previous Tab"), "<Control>Page_Up", N_ ("Switch to previous tab"), G_CALLBACK (terminal_window_action_prev_tab), },
     { "next-tab", GTK_STOCK_GO_FORWARD, N_ ("_Next Tab"), "<Control>Page_Down", N_ ("Switch to next tab"), G_CALLBACK (terminal_window_action_next_tab), },
     { "move-tab-left", NULL, N_ ("Move Tab _Left"), NULL, NULL, G_CALLBACK (terminal_window_action_move_tab_left), },
     { "move-tab-right", NULL, N_ ("Move Tab _Right"), NULL, NULL, G_CALLBACK (terminal_window_action_move_tab_right), },
   { "help-menu", NULL, N_ ("_Help"), NULL, NULL, NULL, },
     { "contents", GTK_STOCK_HELP, N_ ("_Contents"), "F1", N_ ("Display help contents"), G_CALLBACK (terminal_window_action_contents), },
-    { "report-bug", NULL, N_ ("_Report a bug"), NULL, NULL, G_CALLBACK (terminal_window_action_report_bug), },
     { "about", GTK_STOCK_ABOUT, N_ ("_About"), NULL, NULL, G_CALLBACK (terminal_window_action_about), },
   { "input-methods", NULL, N_ ("_Input Methods"), NULL, NULL, NULL, },
 };
@@ -331,7 +278,7 @@ terminal_window_class_init (TerminalWindowClass *klass)
                   G_TYPE_INT, G_TYPE_INT);
 
   /* initialize quark */
-  gomenu_action_quark = g_quark_from_static_string ("gomenu-item");
+  tabs_menu_action_quark = g_quark_from_static_string ("tabs-menu-item");
 }
 
 
@@ -340,10 +287,8 @@ static void
 terminal_window_init (TerminalWindow *window)
 {
   GtkAccelGroup  *accel_group;
-  GtkAction      *action;
   GtkWidget      *vbox;
   gboolean        always_show_tabs;
-  GtkRcStyle     *style;
   GdkScreen      *screen;
   GdkColormap    *colormap;
 
@@ -376,19 +321,10 @@ terminal_window_init (TerminalWindow *window)
   g_signal_connect_after (G_OBJECT (accel_group), "accel-activate",
       G_CALLBACK (terminal_window_accel_activate), window);
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  window->vbox = vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
   gtk_widget_show (vbox);
 
-#if defined(GDK_WINDOWING_X11)
-  /* setup fullscreen mode */
-  if (!gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE)))
-    {
-      action = gtk_action_group_get_action (window->action_group, "fullscreen");
-      gtk_action_set_sensitive (action, FALSE);
-    }
-#endif
-
   /* allocate the notebook for the terminal screens */
   g_object_get (G_OBJECT (window->preferences), "misc-always-show-tabs", &always_show_tabs, NULL);
   window->notebook = g_object_new (GTK_TYPE_NOTEBOOK,
@@ -399,15 +335,9 @@ terminal_window_init (TerminalWindow *window)
                                    "tab-hborder", 0,
                                    "tab-vborder", 0,
                                    NULL);
-  g_object_bind_property (G_OBJECT (window->preferences), "misc-tab-position",
-                          G_OBJECT (window->notebook), "tab-pos",
-                          G_BINDING_SYNC_CREATE);
 
   /* hide the ugly terminal border when tabs are shown */
-  style = gtk_rc_style_new ();
-  style->xthickness = style->ythickness = 0;
-  gtk_widget_modify_style (window->notebook, style);
-  g_object_unref (G_OBJECT (style));
+  terminal_util_set_style_thinkess (window->notebook, 0);
 
   /* set the notebook group id */
   gtk_notebook_set_group (GTK_NOTEBOOK (window->notebook),
@@ -448,6 +378,13 @@ terminal_window_init (TerminalWindow *window)
   window->action_copy = gtk_action_group_get_action (window->action_group, "copy");
   window->action_search_next = gtk_action_group_get_action (window->action_group, "search-next");
   window->action_search_prev = gtk_action_group_get_action (window->action_group, "search-prev");
+  window->action_fullscreen = gtk_action_group_get_action (window->action_group, "fullscreen");
+
+#if defined(GDK_WINDOWING_X11)
+  /* setup fullscreen mode */
+  if (!gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE)))
+    gtk_action_set_sensitive (window->action_fullscreen, FALSE);
+#endif
 }
 
 
@@ -462,7 +399,7 @@ terminal_window_finalize (GObject *object)
   g_object_unref (G_OBJECT (window->ui_manager));
   g_object_unref (G_OBJECT (window->encoding_action));
 
-  g_slist_free (window->gomenu_actions);
+  g_slist_free (window->tabs_menu_actions);
 
   (*G_OBJECT_CLASS (terminal_window_parent_class)->finalize) (object);
 }
@@ -499,18 +436,17 @@ terminal_window_state_event (GtkWidget           *widget,
                              GdkEventWindowState *event)
 {
   TerminalWindow *window = TERMINAL_WINDOW (widget);
-  GtkAction      *action;
   gboolean        fullscreen;
 
   terminal_return_val_if_fail (TERMINAL_IS_WINDOW (window), FALSE);
 
   /* update the fullscreen action if the fullscreen state changed by the wm */
-  if ((event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) != 0)
+  if ((event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) != 0
+      && gtk_widget_get_visible (widget))
     {
       fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
-      action = gtk_action_group_get_action (window->action_group, "fullscreen");
-      if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != fullscreen)
-        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), fullscreen);
+      if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)) != fullscreen)
+        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (window->action_fullscreen), fullscreen);
     }
 
   if (GTK_WIDGET_CLASS (terminal_window_parent_class)->window_state_event != NULL)
@@ -681,7 +617,8 @@ terminal_window_set_size_force_grid (TerminalWindow *window,
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   /* required to get the char height/width right */
-  if (gtk_widget_get_realized (GTK_WIDGET (screen)))
+  if (gtk_widget_get_realized (GTK_WIDGET (screen))
+      && !window->drop_down)
     {
       terminal_screen_force_resize_window (screen, GTK_WINDOW (window),
                                            force_grid_width, force_grid_height);
@@ -762,7 +699,7 @@ terminal_window_update_actions (TerminalWindow *window)
       gtk_action_set_sensitive (window->action_search_prev, can_search);
 
       /* update the "Go" menu */
-      action = g_object_get_qdata (G_OBJECT (window->active), gomenu_action_quark);
+      action = g_object_get_qdata (G_OBJECT (window->active), tabs_menu_action_quark);
       if (G_LIKELY (action != NULL))
         gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
     }
@@ -771,31 +708,31 @@ terminal_window_update_actions (TerminalWindow *window)
 
 
 static void
-terminal_window_rebuild_gomenu (TerminalWindow *window)
+terminal_window_rebuild_tabs_menu (TerminalWindow *window)
 {
   gint            npages, n;
   GtkWidget      *page;
   GSList         *group = NULL;
   GtkRadioAction *radio_action;
-  gchar           name[100];
+  gchar           name[50];
   GSList         *lp;
 
-  if (window->gomenu_merge_id != 0)
+  if (window->tabs_menu_merge_id != 0)
     {
       /* remove merge id */
-      gtk_ui_manager_remove_ui (window->ui_manager, window->gomenu_merge_id);
+      gtk_ui_manager_remove_ui (window->ui_manager, window->tabs_menu_merge_id);
 
       /* drop all the old accels from the action group */
-      for (lp = window->gomenu_actions; lp != NULL; lp = lp->next)
+      for (lp = window->tabs_menu_actions; lp != NULL; lp = lp->next)
         gtk_action_group_remove_action (window->action_group, GTK_ACTION (lp->data));
 
-      g_slist_free (window->gomenu_actions);
-      window->gomenu_actions = NULL;
+      g_slist_free (window->tabs_menu_actions);
+      window->tabs_menu_actions = NULL;
     }
 
   /* create a new merge id */
-  window->gomenu_merge_id = gtk_ui_manager_new_merge_id (window->ui_manager);
-  terminal_assert (window->gomenu_actions == NULL);
+  window->tabs_menu_merge_id = gtk_ui_manager_new_merge_id (window->ui_manager);
+  terminal_assert (window->tabs_menu_actions == NULL);
 
   /* walk the tabs */
   npages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->notebook));
@@ -816,23 +753,30 @@ terminal_window_rebuild_gomenu (TerminalWindow *window)
       gtk_action_group_add_action (window->action_group, GTK_ACTION (radio_action));
       g_signal_connect (G_OBJECT (radio_action), "activate",
           G_CALLBACK (terminal_window_action_goto_tab), window->notebook);
-      gtk_action_set_sensitive (GTK_ACTION (radio_action), TRUE);
 
       /* connect action to the page so we can active it when a tab is switched */
-      g_object_set_qdata_full (G_OBJECT (page), gomenu_action_quark,
+      g_object_set_qdata_full (G_OBJECT (page), tabs_menu_action_quark,
                                radio_action, g_object_unref);
 
       /* add action in the menu */
-      gtk_ui_manager_add_ui (window->ui_manager, window->gomenu_merge_id,
-                             "/main-menu/go-menu/placeholder-tab-items",
+      gtk_ui_manager_add_ui (window->ui_manager, window->tabs_menu_merge_id,
+                             "/main-menu/tabs-menu/placeholder-tab-items",
                              name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
 
+      if (npages > 1)
+        {
+          /* add to right-click tab menu */
+          gtk_ui_manager_add_ui (window->ui_manager, window->tabs_menu_merge_id,
+                                 "/tab-menu/tabs-menu/placeholder-tab-items",
+                                 name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+        }
+
       /* set an accelerator path */
       g_snprintf (name, sizeof (name), "<Actions>/terminal-window/goto-tab-%d", n + 1);
       gtk_action_set_accel_path (GTK_ACTION (radio_action), name);
 
       /* store */
-      window->gomenu_actions = g_slist_prepend (window->gomenu_actions, radio_action);
+      window->tabs_menu_actions = g_slist_prepend (window->tabs_menu_actions, radio_action);
     }
 }
 
@@ -891,35 +835,7 @@ terminal_window_notebook_page_reordered (GtkNotebook     *notebook,
 {
 
   /* Regenerate the "Go" menu */
-  terminal_window_rebuild_gomenu (window);
-}
-
-
-
-static void
-terminal_window_notebook_show_tabs (TerminalWindow *window)
-{
-  GtkNotebook *notebook = GTK_NOTEBOOK (window->notebook);
-  gboolean     show_tabs = TRUE;
-  gint         npages;
-
-  /* set the visibility of the tabs */
-  npages = gtk_notebook_get_n_pages (notebook);
-  if (npages < 2)
-    g_object_get (G_OBJECT (window->preferences),
-                  "misc-always-show-tabs", &show_tabs, NULL);
-
-  if (gtk_notebook_get_show_tabs (notebook) != show_tabs)
-    {
-      /* store size */
-      terminal_window_size_push (window);
-
-      /* show or hdie the tabs */
-      gtk_notebook_set_show_tabs (notebook, show_tabs);
-
-      /* update the window geometry */
-      terminal_window_size_pop (window);
-    }
+  terminal_window_rebuild_tabs_menu (window);
 }
 
 
@@ -960,6 +876,12 @@ terminal_window_notebook_page_added (GtkNotebook    *notebook,
       /* show the tabs when needed */
       terminal_window_notebook_show_tabs (window);
     }
+  else if (G_UNLIKELY (window->drop_down))
+    {
+      /* try to calculate a decent grid size based on the info we have now */
+      terminal_window_dropdown_get_size (TERMINAL_WINDOW_DROPDOWN (window), screen, &w, &h);
+      terminal_screen_set_size (screen, w, h);
+    }
   else
     {
       /* force a screen size, needed for misc-default-geometry */
@@ -968,7 +890,7 @@ terminal_window_notebook_page_added (GtkNotebook    *notebook,
     }
 
   /* regenerate the "Go" menu */
-  terminal_window_rebuild_gomenu (window);
+  terminal_window_rebuild_tabs_menu (window);
 }
 
 
@@ -985,7 +907,7 @@ terminal_window_notebook_page_removed (GtkNotebook    *notebook,
   terminal_return_if_fail (TERMINAL_IS_WINDOW (window));
 
   /* unset the go menu item */
-  g_object_set_qdata (G_OBJECT (child), gomenu_action_quark, NULL);
+  g_object_set_qdata (G_OBJECT (child), tabs_menu_action_quark, NULL);
 
   /* disconnect signals */
   g_signal_handlers_disconnect_by_func (G_OBJECT (child),
@@ -1010,7 +932,7 @@ terminal_window_notebook_page_removed (GtkNotebook    *notebook,
       terminal_window_notebook_show_tabs (window);
 
       /* regenerate the "Go" menu */
-      terminal_window_rebuild_gomenu (window);
+      terminal_window_rebuild_tabs_menu (window);
     }
 }
 
@@ -1414,30 +1336,41 @@ terminal_window_action_select_all (GtkAction      *action,
 
 
 
+static void
+terminal_window_action_prefs_died (gpointer  user_data,
+                                   GObject  *where_the_object_was)
+{
+  TerminalWindow *window = TERMINAL_WINDOW (user_data);
+
+  window->preferences_dialog = NULL;
+  window->n_child_windows--;
+
+  if (window->drop_down)
+    terminal_util_activate_window (GTK_WINDOW (window));
+}
+
+
+
 static void
 terminal_window_action_prefs (GtkAction      *action,
                               TerminalWindow *window)
 {
-  /* check if we already have a preferences dialog instance */
-  if (G_UNLIKELY (window->preferences_dialog != NULL))
+  if (window->preferences_dialog == NULL)
     {
-      /* move to the current screen and make transient for this window */
-      gtk_window_set_screen (GTK_WINDOW (window->preferences_dialog), gtk_widget_get_screen (GTK_WIDGET (window)));
-      gtk_window_set_transient_for (GTK_WINDOW (window->preferences_dialog), GTK_WINDOW (window));
-
-      /* present the preferences dialog on the current workspace */
-      gtk_window_present (GTK_WINDOW (window->preferences_dialog));
-    }
-  else
-    {
-      /* allocate a new preferences dialog instance */
-      window->preferences_dialog = terminal_preferences_dialog_new (GTK_WINDOW (window));
+      window->preferences_dialog = terminal_preferences_dialog_new (window->drop_down);
       if (G_LIKELY (window->preferences_dialog != NULL))
         {
-          g_object_add_weak_pointer (G_OBJECT (window->preferences_dialog), (gpointer) &window->preferences_dialog);
-          gtk_widget_show (window->preferences_dialog);
+          window->n_child_windows++;
+          g_object_weak_ref (G_OBJECT (window->preferences_dialog),
+                             terminal_window_action_prefs_died, window);
         }
     }
+
+  if (window->preferences_dialog != NULL)
+    {
+      gtk_window_set_transient_for (GTK_WINDOW (window->preferences_dialog), GTK_WINDOW (window));
+      gtk_window_present (GTK_WINDOW (window->preferences_dialog));
+    }
 }
 
 
@@ -1446,8 +1379,6 @@ static void
 terminal_window_action_show_menubar (GtkToggleAction *action,
                                      TerminalWindow  *window)
 {
-  GtkWidget *vbox;
-
   terminal_return_if_fail (GTK_IS_UI_MANAGER (window->ui_manager));
 
   terminal_window_size_push (window);
@@ -1456,10 +1387,9 @@ terminal_window_action_show_menubar (GtkToggleAction *action,
     {
       if (G_LIKELY (window->menubar == NULL))
         {
-          vbox = gtk_bin_get_child (GTK_BIN (window));
           window->menubar = gtk_ui_manager_get_widget (window->ui_manager, "/main-menu");
-          gtk_box_pack_start (GTK_BOX (vbox), window->menubar, FALSE, FALSE, 0);
-          gtk_box_reorder_child (GTK_BOX (vbox), window->menubar, 0);
+          gtk_box_pack_start (GTK_BOX (window->vbox), window->menubar, FALSE, FALSE, 0);
+          gtk_box_reorder_child (GTK_BOX (window->vbox), window->menubar, 0);
         }
 
       gtk_widget_show (window->menubar);
@@ -1478,8 +1408,6 @@ static void
 terminal_window_action_show_toolbar (GtkToggleAction *action,
                                      TerminalWindow  *window)
 {
-  GtkWidget *vbox;
-
   terminal_return_if_fail (GTK_IS_UI_MANAGER (window->ui_manager));
   terminal_return_if_fail (GTK_IS_ACTION_GROUP (window->action_group));
 
@@ -1489,18 +1417,16 @@ terminal_window_action_show_toolbar (GtkToggleAction *action,
     {
       if (window->toolbar == NULL)
         {
-          vbox = gtk_bin_get_child (GTK_BIN (window));
           window->toolbar = gtk_ui_manager_get_widget (window->ui_manager, "/main-toolbar");
-          gtk_box_pack_start (GTK_BOX (vbox), window->toolbar, FALSE, FALSE, 0);
-          gtk_box_reorder_child (GTK_BOX (vbox), window->toolbar, window->menubar != NULL ? 1 : 0);
+          gtk_box_pack_start (GTK_BOX (window->vbox), window->toolbar, FALSE, FALSE, 0);
+          gtk_box_reorder_child (GTK_BOX (window->vbox), window->toolbar, window->menubar != NULL ? 1 : 0);
         }
 
       gtk_widget_show (window->toolbar);
     }
   else if (window->toolbar != NULL)
     {
-      gtk_widget_destroy (window->toolbar);
-      window->toolbar = NULL;
+      gtk_widget_hide (window->toolbar);
     }
 
   terminal_window_size_pop (window);
@@ -1616,9 +1542,9 @@ terminal_window_action_goto_tab (GtkRadioAction *action,
 
 
 static void
-title_dialog_response (GtkWidget *dialog,
-                       gint       response,
-                       GBinding  *title_binding)
+title_dialog_response (GtkWidget      *dialog,
+                       gint            response,
+                       TerminalWindow *window)
 {
   /* check if we should open the user manual */
   if (response == GTK_RESPONSE_HELP)
@@ -1628,8 +1554,12 @@ title_dialog_response (GtkWidget *dialog,
     }
   else
     {
+      /* need for hiding on focus */
+      if (window->drop_down)
+        terminal_util_activate_window (GTK_WINDOW (window));
+
       /* close the dialog */
-      g_object_unref (title_binding);
+      window->n_child_windows--;
       gtk_widget_destroy (dialog);
     }
 }
@@ -1637,9 +1567,11 @@ title_dialog_response (GtkWidget *dialog,
 
 
 static void
-title_dialog_quit (GtkWidget *dialog)
+title_dialog_clear (GtkWidget            *entry,
+                    GtkEntryIconPosition  icon_pos)
 {
-  gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+  if (icon_pos == GTK_ENTRY_ICON_SECONDARY)
+    gtk_entry_set_text (GTK_ENTRY (entry), "");
 }
 
 
@@ -1648,54 +1580,51 @@ static void
 terminal_window_action_set_title (GtkAction      *action,
                                   TerminalWindow *window)
 {
-  AtkRelationSet   *relations;
-  AtkRelation      *relation;
-  AtkObject        *object;
-  GtkWidget        *dialog;
-  GtkWidget        *box;
-  GtkWidget        *label;
-  GtkWidget        *entry;
-  GBinding         *title_binding;
+  AtkObject *object;
+  GtkWidget *dialog;
+  GtkWidget *box;
+  GtkWidget *label;
+  GtkWidget *entry;
 
   if (G_LIKELY (window->active != NULL))
     {
+      window->n_child_windows++;
+
       dialog = gtk_dialog_new_with_buttons (Q_("Window Title|Set Title"),
                                             GTK_WINDOW (window),
-                                            GTK_DIALOG_DESTROY_WITH_PARENT
-                                            | GTK_DIALOG_NO_SEPARATOR,
+                                            GTK_DIALOG_DESTROY_WITH_PARENT,
                                             GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                             GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                             NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
       box = gtk_hbox_new (FALSE, 12);
       gtk_container_set_border_width (GTK_CONTAINER (box), 6);
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box, TRUE, TRUE, 0);
       gtk_widget_show (box);
 
-      label = gtk_label_new (_("Title:"));
+      label = gtk_label_new_with_mnemonic (_("_Title:"));
       gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
       gtk_widget_show (label);
 
       entry = gtk_entry_new ();
       gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
-      g_signal_connect_swapped (G_OBJECT (entry), "activate",
-                                G_CALLBACK (title_dialog_quit), dialog);
+      gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+      gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+      gtk_entry_set_icon_from_stock (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+      g_signal_connect (G_OBJECT (entry), "icon-release", G_CALLBACK (title_dialog_clear), NULL);
       gtk_widget_show (entry);
 
       /* set Atk description and label relation for the entry */
       object = gtk_widget_get_accessible (entry);
       atk_object_set_description (object, _("Enter the title for the current terminal tab"));
-      relations = atk_object_ref_relation_set (gtk_widget_get_accessible (label));
-      relation = atk_relation_new (&object, 1, ATK_RELATION_LABEL_FOR);
-      atk_relation_set_add (relations, relation);
-      g_object_unref (G_OBJECT (relation));
 
-      title_binding = g_object_bind_property (G_OBJECT (window->active), "custom-title",
-                                              G_OBJECT (entry), "text",
-                                              G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+      g_object_bind_property (G_OBJECT (window->active), "custom-title",
+                              G_OBJECT (entry), "text",
+                              G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
       g_signal_connect (G_OBJECT (dialog), "response",
-                        G_CALLBACK (title_dialog_response), title_binding);
+                        G_CALLBACK (title_dialog_response), window);
 
       gtk_widget_show (dialog);
     }
@@ -1742,7 +1671,12 @@ terminal_window_action_search_response (GtkWidget      *dialog,
     }
   else
     {
+      /* need for hiding on focus */
+      if (window->drop_down)
+        terminal_util_activate_window (GTK_WINDOW (window));
+
       /* hide dialog */
+      window->n_child_windows--;
       gtk_widget_hide (dialog);
     }
 
@@ -1763,9 +1697,14 @@ terminal_window_action_search (GtkAction      *action,
       window->search_dialog = terminal_search_dialog_new (GTK_WINDOW (window));
       g_signal_connect (G_OBJECT (window->search_dialog), "response",
           G_CALLBACK (terminal_window_action_search_response), window);
-      gtk_widget_show_all (window->search_dialog);
+      g_signal_connect (G_OBJECT (window->search_dialog), "delete-event",
+          G_CALLBACK (gtk_widget_hide_on_delete), NULL);
     }
 
+  /* increase child counter */
+  if (!gtk_widget_get_visible (window->search_dialog))
+    window->n_child_windows++;
+
   terminal_search_dialog_present (TERMINAL_SEARCH_DIALOG (window->search_dialog));
 }
 
@@ -1814,16 +1753,6 @@ terminal_window_action_reset_and_clear (GtkAction       *action,
 
 
 
-static void
-terminal_window_action_report_bug (GtkAction       *action,
-                                   TerminalWindow  *window)
-{
-  /* open the "Support" section of the user manual */
-  xfce_dialog_show_help (GTK_WINDOW (window), "terminal", "support", NULL);
-}
-
-
-
 static void
 terminal_window_action_contents (GtkAction       *action,
                                  TerminalWindow  *window)
@@ -1839,7 +1768,7 @@ terminal_window_action_about (GtkAction      *action,
                               TerminalWindow *window)
 {
   /* display the about dialog */
-  terminal_dialogs_show_about (GTK_WINDOW (window));
+  terminal_util_show_about_dialog (GTK_WINDOW (window));
 }
 
 
@@ -1876,9 +1805,8 @@ terminal_window_new (const gchar       *role,
                 NULL);
 
   /* setup full screen */
-  action = gtk_action_group_get_action (window->action_group, "fullscreen");
-  if (fullscreen && gtk_action_is_sensitive (action))
-    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  if (fullscreen && gtk_action_is_sensitive (window->action_fullscreen))
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (window->action_fullscreen), TRUE);
 
   /* setup menubar visibility */
   if (G_LIKELY (menubar != TERMINAL_VISIBILITY_DEFAULT))
@@ -1898,6 +1826,11 @@ terminal_window_new (const gchar       *role,
   action = gtk_action_group_get_action (window->action_group, "show-borders");
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_borders);
 
+  /* property that is not suitable for init */
+  g_object_bind_property (G_OBJECT (window->preferences), "misc-tab-position",
+                          G_OBJECT (window->notebook), "tab-pos",
+                          G_BINDING_SYNC_CREATE);
+
   return GTK_WIDGET (window);
 }
 
@@ -1959,6 +1892,40 @@ terminal_window_get_active (TerminalWindow *window)
 
 
 
+void
+terminal_window_notebook_show_tabs (TerminalWindow *window)
+{
+  GtkNotebook *notebook = GTK_NOTEBOOK (window->notebook);
+  gboolean     show_tabs = TRUE;
+  gint         npages;
+
+  terminal_return_if_fail (TERMINAL_IS_WINDOW (window));
+
+  /* check preferences */
+  npages = gtk_notebook_get_n_pages (notebook);
+  if (npages < 2)
+    {
+      g_object_get (G_OBJECT (window->preferences),
+                    window->drop_down ? "dropdown-always-show-tabs" :
+                    "misc-always-show-tabs", &show_tabs, NULL);
+    }
+
+  /* set the visibility of the tabs */
+  if (gtk_notebook_get_show_tabs (notebook) != show_tabs)
+    {
+      /* store size */
+      terminal_window_size_push (window);
+
+      /* show or hdie the tabs */
+      gtk_notebook_set_show_tabs (notebook, show_tabs);
+
+      /* update the window geometry */
+      terminal_window_size_pop (window);
+    }
+}
+
+
+
 /**
  * terminal_window_get_restart_command:
  * @window  : A #TerminalWindow.
@@ -1998,8 +1965,7 @@ terminal_window_get_restart_command (TerminalWindow *window)
   if (G_LIKELY (role != NULL))
     result = g_slist_prepend (result, g_strdup_printf ("--role=%s", role));
 
-  action = gtk_action_group_get_action (window->action_group, "fullscreen");
-  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
+  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
     result = g_slist_prepend (result, g_strdup ("--fullscreen"));
 
   action = gtk_action_group_get_action (window->action_group, "show-menubar");
